@@ -70,7 +70,11 @@ static const wchar_t kRecursedIDChar = 'R';
 static const wchar_t *kRecursedPostCharSet = L"0-";
 
 static const wchar_t *kDefaultArchiveType = L"7z";
+#ifdef _WIN32
 static const wchar_t *kSFXExtension = L"exe";
+#else
+static const wchar_t *kSFXExtension = L"";
+#endif
 
 namespace NRecursedPostCharIndex {
   enum EEnum 
@@ -234,7 +238,7 @@ static bool AddNameToCensor(NWildcard::CCensor &wildcardCensor,
       recursed = false;
       break;
   }
-  wildcardCensor.AddItem(name, include, recursed);
+  wildcardCensor.AddItem(include, name, recursed);
   return true;
 }
 
@@ -380,6 +384,72 @@ static void AddSwitchWildCardsToCensor(NWildcard::CCensor &wildcardCensor,
   }
 }
 
+#ifdef _WIN32
+
+// This code converts all short file names to long file names.
+
+static void ConvertToLongName(const UString &prefix, UString &name)
+{
+  if (name.IsEmpty() || DoesNameContainWildCard(name))
+    return;
+  NFind::CFileInfoW fileInfo;
+  if (NFind::FindFile(prefix + name, fileInfo))
+    name = fileInfo.Name;
+}
+
+static void ConvertToLongNames(const UString &prefix, CObjectVector<NWildcard::CItem> &items)
+{
+  for (int i = 0; i < items.Size(); i++)
+  {
+    NWildcard::CItem &item = items[i];
+    if (item.Recursive || item.PathParts.Size() != 1)
+      continue;
+    ConvertToLongName(prefix, item.PathParts.Front());
+  }
+}
+
+static void ConvertToLongNames(const UString &prefix, NWildcard::CCensorNode &node)
+{
+  ConvertToLongNames(prefix, node.IncludeItems);
+  ConvertToLongNames(prefix, node.ExcludeItems);
+  int i;
+  for (i = 0; i < node.SubNodes.Size(); i++)
+    ConvertToLongName(prefix, node.SubNodes[i].Name);
+  // mix folders with same name
+  for (i = 0; i < node.SubNodes.Size(); i++)
+  {
+    NWildcard::CCensorNode &nextNode1 = node.SubNodes[i];
+    for (int j = i + 1; j < node.SubNodes.Size();)
+    {
+      const NWildcard::CCensorNode &nextNode2 = node.SubNodes[j];
+      if (nextNode1.Name.CompareNoCase(nextNode2.Name) == 0)
+      {
+        nextNode1.IncludeItems += nextNode2.IncludeItems;
+        nextNode1.ExcludeItems += nextNode2.ExcludeItems;
+        node.SubNodes.Delete(j);
+      }
+      else
+        j++;
+    }
+  }
+  for (i = 0; i < node.SubNodes.Size(); i++)
+  {
+    NWildcard::CCensorNode &nextNode = node.SubNodes[i];
+    ConvertToLongNames(prefix + nextNode.Name + wchar_t(NFile::NName::kDirDelimiter), nextNode); 
+  }
+}
+
+static void ConvertToLongNames(NWildcard::CCensor &censor)
+{
+  for (int i = 0; i < censor.Pairs.Size(); i++)
+  {
+    NWildcard::CPair &pair = censor.Pairs[i];
+    ConvertToLongNames(pair.Prefix, pair.Head);
+  }
+}
+
+#endif
+
 static NUpdateArchive::NPairAction::EEnum GetUpdatePairActionType(int i)
 {
   switch(i)
@@ -486,7 +556,7 @@ static bool ParseComplexSize(const UString &src, UInt64 &result)
   const wchar_t *start = s;
   const wchar_t *end;
   UInt64 number = ConvertStringToUInt64(start, &end);
-  int numDigits = end - start;
+  int numDigits = (int)(end - start);
   if (numDigits == 0 || s.Length() > numDigits + 1)
     return false;
   if (s.Length() == numDigits)
@@ -631,7 +701,7 @@ static void SetArchiveType(const UString &archiveType,
 
 CArchiveCommandLineParser::CArchiveCommandLineParser(): parser(kNumSwitches) {}
 
-void CArchiveCommandLineParser::Parse1(UStringVector commandStrings,
+void CArchiveCommandLineParser::Parse1(const UStringVector &commandStrings,
     CArchiveCommandLineOptions &options)
 {
   try
@@ -727,7 +797,7 @@ void CArchiveCommandLineParser::Parse2(CArchiveCommandLineOptions &options)
 
     bool directlyAddArchiveName = false;
     if (thereIsArchiveName) {
-      if ((options.ArchiveName.Find(L"*") == -1) && (options.ArchiveName.Find(L"?") == -1)) {
+      if ((options.ArchiveName.Find(kUniversalWildcard) == -1) && (options.ArchiveName.Find(L"?") == -1)) {
         // no wildcard => no need to scan
         directlyAddArchiveName = true;
       } else {
@@ -735,9 +805,18 @@ void CArchiveCommandLineParser::Parse2(CArchiveCommandLineOptions &options)
       }
     }
 
+    #ifdef _WIN32
+    ConvertToLongNames(archiveWildcardCensor);
+    #endif
+
     CObjectVector<CDirItem> dirItems;
-    UString path_error; // FIXME
-    EnumerateItems(archiveWildcardCensor, dirItems, NULL,path_error);
+    {
+      UStringVector errorPaths;
+      CRecordVector<DWORD> errorCodes;
+      HRESULT res = EnumerateItems(archiveWildcardCensor, dirItems, NULL, errorPaths, errorCodes);
+      if (res != S_OK || errorPaths.Size() > 0)
+        throw "cannot find archive";
+    }
     UStringVector archivePaths;
     int i;
     for (i = 0; i < dirItems.Size(); i++)
@@ -847,6 +926,10 @@ void CArchiveCommandLineParser::Parse2(CArchiveCommandLineOptions &options)
       throw kTerminalOutError;
     if(updateOptions.StdInMode)
       updateOptions.StdInFileName = parser[NKey::kStdIn].PostStrings.Front();
+
+    #ifdef _WIN32
+    ConvertToLongNames(options.WildcardCensor);
+    #endif
   }
   else 
     throw kUserErrorMessage;
